/************************************************************
 * Joseph Zambreno           
 * Department of Electrical and Computer Engineering
 * Iowa State University
 *                        
 * FILE: BubbleSort.c              
 *
 * DESCRIPTION: This program sorts N hard-coded values of
 * different sizes. Compile with -D_SORT_u[X]_ to choose 
 * the different data types and associated inputs.
 *
 * NOTES:
 * 8/24/06 by JAZ::Design created. 
 ************************************************************/

#define N 8

#ifdef _SORT_u8_
#define _VALUE_
typedef unsigned char value_type;
#define MAX_VALUE 0xFF
#endif

#ifdef _SORT_u16_
#define _VALUE_
typedef unsigned short value_type;
#define MAX_VALUE 0xFFFF
#endif

#ifdef _SORT_u32_
#define _VALUE_
typedef unsigned int value_type;
#define MAX_VALUE 0xFFFFFFFF
#endif

#ifdef _SORT_u64_
#define _VALUE_
typedef unsigned long long value_type;
#define MAX_VALUE 0xFFFFFFFF
#endif

#ifndef _VALUE_
#define _VALUE_
typedef unsigned char value_type;
#define MAX_VALUE 0xFF
#endif

#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>

int main() {
  int i, j;
  value_type *inputs;
  value_type temp_value;
  struct timeval before;
  struct timeval after;

  inputs = (value_type *)malloc(N*sizeof(value_type));
  for (i = 0; i < N; i++) {
    inputs[i] = rand() % MAX_VALUE;
  }

  #ifdef _DEBUG_
  printf("\nBefore sorting: inputs[%d] = { ", N);
  for (i = 0; i < N; i++) {
    printf("0x%x ", inputs[i]);
  }
  printf("}\n");
  #endif

  gettimeofday(&before, NULL);
  for (i = N-1; i >= 0; i--) {
    for (j = 1; j <= i; j++) {
      if (inputs[j-1] > inputs[j]) {
	temp_value = inputs[j-1];
	inputs[j-1] = inputs[j];
	inputs[j] = temp_value;
      }
    }
  }
  gettimeofday(&after, NULL);
  
  #ifdef _DEBUG_
  printf("After sorting:  inputs[%d] = { ", N);
  for (i = 0; i < N; i++) {
    printf("0x%x ", inputs[i]);
  }
  printf("}\n\n");
  #endif


  /* JAZ::Don't look at this bit of nastiness */
  printf("Operation required ");
  if ((after.tv_sec - before.tv_sec) > 1) {
    if (after.tv_usec < before.tv_usec) {
      printf("%u.%08u seconds\n", after.tv_sec-before.tv_sec,
	     1000000-before.tv_usec+after.tv_usec);
    }
    else {
      printf("%u.%08u seconds\n", after.tv_sec-before.tv_sec,
	     after.tv_usec-before.tv_usec);
    }
  }
  else if (after.tv_sec > before.tv_sec) {
        if (after.tv_usec < before.tv_usec) {
	  printf("%u microseconds\n", 1000000-before.tv_usec+after.tv_usec);
	}
	else {
	  printf("%u.%08u seconds\n", after.tv_sec-before.tv_sec,
		 after.tv_usec-before.tv_usec);
	}
  }
  else {
    printf("%u microseconds\n", after.tv_usec-before.tv_usec);
  }

  return 0;
}
